@@ -3,41 +3,38 @@
 The initial motivation for this wrapper is to abstract away
 the mturk functionality from the shell
 """
-from __future__ import print_function
-from __future__ import absolute_import
-
+from __future__ import generator_stop
 from functools import wraps
-from six import string_types
-
-from .utils import *
 from .models import Participant, Hit
 from .db import db_session, init_db
 from .psiturk_exceptions import *
 from .psiturk_statuses import *
 from .psiturk_config import PsiturkConfig
-from .psiturk_org_services import PsiturkOrgServices
 from .amt_services import MTurkServices
-from sqlalchemy import and_
 import sqlalchemy as sa
 import datetime
 import random
 import string
-import os
-import json
-import sys
 from builtins import object
 from builtins import range
 from builtins import str
-from future import standard_library
-standard_library.install_aliases()
+
+try:
+    import gnureadline as readline
+except ImportError:
+    import readline
 
 
 class WrapperResponse(object):
-    def __init__(self, status=None, message='', data={}, operation='', **kwargs):
+    def __init__(self, status=None, message='', data: dict = None,
+                 operation='', **kwargs):
+        self.dict_keys = ['status', 'success', 'message', 'data', 'operation']
         self.status = status
         self.message = message
-        self.data = data
+        self.data = data if data is not None else {}
         self.operation = operation
+        self.mode = None
+        self.sandbox = None
         for k, v in kwargs.items():
             setattr(self, k, v)
 
@@ -57,63 +54,69 @@ class WrapperResponse(object):
 
         return 'Response({})'.format(details)
 
+    def to_dict(self):
+        _dict = {}
+        for key in self.dict_keys:
+            _dict[key] = getattr(self, key)
+        return _dict
+
 
 class WrapperResponseSuccess(WrapperResponse):
     def __init__(self, *args, **kwargs):
         super(WrapperResponseSuccess, self).__init__(
             *args, status='success', success=True, **kwargs)
 
+
 class WrapperResponseError(WrapperResponse):
     def __init__(self, *args, **kwargs):
         super(WrapperResponseError, self).__init__(
             *args, status='error', success=False, **kwargs)
+
+        self.dict_keys.append('exception')
         self.exception = kwargs.pop('exception', None)
-            
+
 
 def amt_services_wrapper_response(func):
+    """Return amt_services_wrapper_response decorator."""
     @wraps(func)
     def wrapper(self, *args, **kwargs):
         try:
             response = func(self, *args, **kwargs)
-            if isinstance(response, WrapperResponseSuccess): # this so that func.__name__ can be set
-                response = response.data
-            elif isinstance(response, WrapperResponseError): # this so that func.__name__ can be set
-                raise response.exception
-            return WrapperResponseSuccess(operation=func.__name__, data=response)
+            if isinstance(response, WrapperResponse):
+                return response
+            if isinstance(response, dict) and 'exception' in response:
+                return WrapperResponseError(
+                    operation=func.__name__,
+                    exception=response.pop('exception'), data=response)
+            return WrapperResponseSuccess(
+                operation=func.__name__,
+                data=response)
         except Exception as e:
             return WrapperResponseError(operation=func.__name__, exception=e)
+
     return wrapper
 
-class MTurkServicesWrapper(object):
 
-    _cached_web_services = None
+class MTurkServicesWrapper(object):
+    """class MTurkServicesWrapper."""
+    
     _cached_dbs_services = None
     _cached_amt_services = None
-
-    @property
-    def web_services(self):
-        if not self._cached_web_services:
-            self._cached_web_services = PsiturkOrgServices(
-                self.config.get('psiTurk Access', 'psiturk_access_key_id'),
-                self.config.get('psiTurk Access', 'psiturk_secret_access_id'))
-        return self._cached_web_services
-
-    def set_web_services(self, web_services):
-        self._cached_web_services = web_services
+    mode = None
 
     @property
     def amt_services(self):
+        """Get amt_services."""
         if not self._cached_amt_services:
             try:
-                self._cached_amt_services = MTurkServices(
-                    self.config.get('AWS Access', 'aws_access_key_id'),
-                    self.config.get('AWS Access', 'aws_secret_access_key'),
-                    self.sandbox)
+                self._cached_amt_services = MTurkServices(mode=self.mode,
+                                                          config=self.config)
             except PsiturkException:
                 raise
         return self._cached_amt_services
 
-    def __init__(self, config=None, web_services=None, sandbox=None):
+    def __init__(self, config=None, web_services=None, mode='sandbox'):
+        """__init__."""
         init_db()
 
         if not config:
@@ -121,33 +124,40 @@ class MTurkServicesWrapper(object):
             config.load_config()
         self.config = config
 
-        if not sandbox:
-            sandbox = config.getboolean(
-                'Shell Parameters', 'launch_in_sandbox_mode')
-        self.sandbox = sandbox
-        
+        self.set_mode(mode)
+
         if web_services:
             self._cached_web_services = web_services
 
-        _ = self.amt_services # may throw an exception. Let it throw!
-
+        _ = self.amt_services  # may throw an exception. Let it throw!
 
     # +-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.
     #   Miscellaneous
     # +-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.
+    @amt_services_wrapper_response
+    def get_mode(self):
+        """Get mode."""
+        return self.mode
 
-    def set_sandbox(self, is_sandbox):
-        self.sandbox = is_sandbox
-        self.amt_services.set_sandbox(is_sandbox)
+    @amt_services_wrapper_response
+    def set_mode(self, mode):
+        """set_mode."""
+        if mode not in ['sandbox', 'live']:
+            raise PsiturkException(f'mode not recognized: {mode}')
+
+        self.mode = mode
+        self.amt_services.set_mode(mode)
+
+        return {'success': True}
 
     def random_id_generator(self, size=6, chars=string.ascii_uppercase +
                             string.digits):
-        ''' Generate random id numbers '''
+        """ Generate random id numbers """
         return ''.join(random.choice(chars) for x in range(size))
 
     @amt_services_wrapper_response
     def amt_balance(self):
-        ''' Get MTurk balance '''
+        """ Get MTurk balance """
         return self.amt_services.check_balance().data
 
     # +-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.
@@ -155,10 +165,9 @@ class MTurkServicesWrapper(object):
     # +-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.+-+.
     @staticmethod
     def add_bonus_info(assignment_dict):
-        " Adds DB-logged worker bonus to worker list data "
+        """ Adds DB-logged worker bonus to worker list data """
         try:
-            unique_id = '{}:{}'.format(
-                assignment_dict['workerId'], assignment_dict['assignmentId'])
+            unique_id = f"{assignment_dict['workerId']}:{assignment_dict['assignmentId']}"
             worker = Participant.query.filter(
                 Participant.uniqueid == unique_id).one()
             assignment_dict['bonus'] = worker.bonus
@@ -168,27 +177,64 @@ class MTurkServicesWrapper(object):
         return assignment_dict
 
     @amt_services_wrapper_response
-    def count_workers(self, codeversion=None, mode='live', status='completed'):
-        ''' 
+    def count_workers(self, codeversion='latest', mode='live', status='completed'):
+        """
         Counts the number of participants in the database who have made it through
         the experiment.
-        '''
-        if not codeversion:
-            codeversion = self.config.get(
-                'Task Parameters', 'experiment_code_version')
+        """
+        if codeversion == 'latest':
+            codeversion = [self.config.get(
+                'task', 'experiment_code_version')]
 
-        worker_count = Participant.query.filter(and_(
-            Participant.status.in_([3, 4, 5, 7]),
-            Participant.codeversion == codeversion,
-            Participant.mode == mode
-        )).count()
+        if status == 'completed':
+            status = [3, 4, 5, 7]
+        query = Participant.query
+        if mode:
+            query = query.filter(Participant.mode == mode)
+        if codeversion:
+            if not isinstance(codeversion, list):
+                codeversion = [codeversion]
+            query = query.filter(Participant.codeversion.in_(codeversion))
+        if status:
+            if not isinstance(status, list):
+                status = [status]
+            query = query.filter(Participant.status.in_(status))
+        worker_count = Participant.count_workers(query=query, group_bys=[])
         return worker_count
 
+    @amt_services_wrapper_response
+    def count_available(self, hits=None):
+        if not hits:
+            hits = self.get_all_hits().data
+        return sum([hit.options['number_assignments_available'] for hit in hits if
+                    hit.options['status'] == 'Assignable'])
+
+    @amt_services_wrapper_response
+    def count_pending(self, hits=None):
+        if not hits:
+            hits = self.get_all_hits().data
+        return sum([hit.options['number_assignments_pending'] for hit in hits if
+                    hit.options['status'] in ['Assignable', 'Unassignable']])
+
+    @amt_services_wrapper_response
+    def count_maybe_will_complete(self, hits=None):
+        maybe_will_complete_count = 0
+        if not hits:
+            hits = self.get_all_hits().data
+        for hit in hits:
+            status = hit.options['status']
+            if status == 'Assignable':
+                maybe_will_complete_count += hit.options['number_assignments_pending'] + \
+                                             hit.options['number_assignments_available']
+            elif status == 'Unassignable':
+                maybe_will_complete_count += hit.options['number_assignments_pending']
+        return maybe_will_complete_count
+
     @amt_services_wrapper_response
     def get_assignments(self, hit_ids=None, assignment_status=None, all_studies=False):
-        '''
+        """
         assignment_status, if set, can be one of `Submitted`, `Approved`, or `Rejected`
-        '''
+        """
         if not all_studies and assignment_status != 'Rejected':
             p_query = Participant.query
             p_query = p_query.filter(~Participant.uniqueid.contains('debug'))
@@ -225,26 +271,26 @@ class MTurkServicesWrapper(object):
         return attempts_were_made
 
     def _try_fetch_local_assignment(self, try_this, with_psiturk_status=None):
-        '''
+        """
         Can accept either an assignment_id or the return of a mturk boto grab...
-        '''
+        """
         query = Participant.query.order_by(Participant.beginhit.desc())
 
         if with_psiturk_status:
             query = query.filter(Participant.status == with_psiturk_status)
-            
-        if isinstance(try_this, string_types):  # then assume that it's an assignment_id
+
+        if isinstance(try_this, str):  # then assume that it's an assignment_id
             assignment_id = try_this
             query = query.filter(Participant.assignmentid == assignment_id)
 
         elif isinstance(try_this, dict):  # then assume that it's a return from mturk
             assignment = try_this
             assignment_id = assignment['assignmentId']
-            query = query.filter(Participant.workerid == assignment['workerId'])\
+            query = query.filter(Participant.workerid == assignment['workerId']) \
                 .filter(Participant.assignmentid == assignment_id)
         else:
-            raise PsiturkException(message='Unrecognized `try_this` value-type: {}'.format(type(try_this)))
-        
+            raise PsiturkException('Unrecognized `try_this` value-type: {}'.format(type(try_this)))
+
         local_assignment = query.first()
         if local_assignment:
             return local_assignment
@@ -277,7 +323,7 @@ class MTurkServicesWrapper(object):
             else:
                 response = self.amt_services.approve_assignment(assignment_id)
                 if response.success:
-                    return True
+                    return {'success': True}
                 else:
                     raise response.exception
 
@@ -288,13 +334,14 @@ class MTurkServicesWrapper(object):
                 assignment_status='Submitted', hit_ids=[hit_id]).data
             results = []
             for assignment in assignments:
-                results.append(self.approve_mturk_assignment(assignment, ignore_local_not_found=all_studies))
+                results.append(
+                    self.approve_mturk_assignment(assignment, ignore_local_not_found=all_studies))
         else:
             assignments = self._get_local_submitted_assignments(hit_id=hit_id)
             results = []
             for assignment in assignments:
                 results.append(self.approve_local_assignment(assignment))
-        return {'results': results}
+        return {'results': results, 'hit_id': hit_id}
 
     def _get_local_submitted_assignments(self, hit_id=None):
         query = Participant.query.filter(Participant.status == SUBMITTED)
@@ -315,7 +362,8 @@ class MTurkServicesWrapper(object):
             assignment_status="Submitted").data
         results = []
         for assignment in assignments:
-            results.append(self.approve_mturk_assignment(assignment, ignore_local_not_found=ignore_local_not_found))
+            results.append(self.approve_mturk_assignment(assignment,
+                                                         ignore_local_not_found=ignore_local_not_found))
         return results
 
     @amt_services_wrapper_response
@@ -330,19 +378,17 @@ class MTurkServicesWrapper(object):
             db_session.commit()
             return {'assignment_id': assignment_id}
         except Exception as e:
-            return WrapperResponseError(exception=e, data={
-                'assignment_id': assignment_id,
-            })
+            return {'exception': e, 'assignment': assignment}
 
     @amt_services_wrapper_response
     def approve_mturk_assignment(self, assignment, ignore_local_not_found=False):
-        ''' Approve assignment '''
+        """ Approve assignment """
         assignment_id = assignment['assignmentId']
 
         found_assignment = False
-        parts = Participant.query.\
-            filter(Participant.assignmentid == assignment_id).\
-            filter(Participant.status.in_([3, 4])).\
+        parts = Participant.query. \
+            filter(Participant.assignmentid == assignment_id). \
+            filter(Participant.status.in_([3, 4])). \
             all()
         # Iterate through all the people who completed this assignment.
         # This should be one person, and it should match the person who
@@ -364,7 +410,7 @@ class MTurkServicesWrapper(object):
             if ignore_local_not_found:
                 response = self.amt_services.approve_assignment(assignment_id)
                 if response.success:
-                    pass # yay
+                    pass  # yay
                 else:
                     raise response.exception
             else:
@@ -398,7 +444,7 @@ class MTurkServicesWrapper(object):
         if not response.success:
             raise response.exception
         else:
-            return True
+            return {'success': True}
 
     @amt_services_wrapper_response
     def unreject_assignments_for_hit(self, hit_id, all_studies=False):
@@ -423,16 +469,16 @@ class MTurkServicesWrapper(object):
 
     @amt_services_wrapper_response
     def unreject_assignment(self, assignment_id, all_studies=False):
-        ''' Unreject assignment '''
+        """ Unreject assignment """
         response = self.amt_services.unreject_assignment(assignment_id)
         result = {}
         if not response.success:
             raise response.exception
         else:
-            message='unrejected {}'.format(assignment_id)
+            message = 'unrejected {}'.format(assignment_id)
             try:
-                participant = Participant.query\
-                    .filter(Participant.assignmentid == assignment_id)\
+                participant = Participant.query \
+                    .filter(Participant.assignmentid == assignment_id) \
                     .order_by(Participant.beginhit.desc()).first()
                 participant.status = CREDITED
                 db_session.add(participant)
@@ -444,11 +490,9 @@ class MTurkServicesWrapper(object):
 
     @amt_services_wrapper_response
     def bonus_all_local_assignments(self, amount, reason, override_bonused_status=False):
-        mode = 'sandbox' if self.sandbox else 'live'
         assignments = Participant.query.filter(Participant.status == CREDITED)\
-            .filter(Participant.mode == mode)\
+            .filter(Participant.mode == self.mode)\
             .all()
-
         results = []
         for assignment in assignments:
             result = self.bonus_local_assignment(
@@ -457,13 +501,16 @@ class MTurkServicesWrapper(object):
         return {'results': results}
 
     @amt_services_wrapper_response
-    def bonus_local_assignment(self, local_assignment, amount, reason, override_bonused_status=False):
+    def bonus_local_assignment(self, local_assignment, amount, reason,
+                               override_bonused_status=False):
         assignment_id = local_assignment.assignmentid
         if local_assignment.status == BONUSED and not override_bonused_status:
             message = 'Participant with assignment_id {} already bonused, and override not set. Not bonusing.'.format(
                 local_assignment.assignmentid)
             raise AssignmentAlreadyBonusedError(message=message)
         if amount == 'auto':
+            if not local_assignment.bonus:
+                return {'exception': NoAutoBonusAmountSetError(), 'assignment_id': assignment_id}
             amount = local_assignment.bonus
 
         response = self.bonus_nonlocal_assignment(
@@ -477,24 +524,25 @@ class MTurkServicesWrapper(object):
         return response
 
     @amt_services_wrapper_response
-    def bonus_assignments_for_hit(self, hit_id, amount, reason, all_studies=False, override_bonused_status=False):
-        '''
+    def bonus_assignments_for_hit(self, hit_id, amount, reason, all_studies=False,
+                                  override_bonused_status=False):
+        """
         Fetch assignments for local hit.
         * If all_studies, try to map them to a local_assignment.
         * If not all_studies, just pull from local db. Already a Participant.
 
         For each, if isinstance Participant, assignment, then send to `bonus_local_assignment`.
         Otherwise, send directly to bonus_assignment. Record the result either way.
-        '''
+        """
         if all_studies:
             mturk_assignments = self.amt_services.get_assignments(
                 assignment_status="Approved", hit_ids=[hit_id]).data
             assignments = self._try_fetch_local_credited_assignments(
                 mturk_assignments)
         else:
-            assignments = Participant.query\
-                .filter(Participant.status.in_([CREDITED, BONUSED]))\
-                .filter(Participant.hitid == hit_id)\
+            assignments = Participant.query \
+                .filter(Participant.status.in_([CREDITED, BONUSED])) \
+                .filter(Participant.hitid == hit_id) \
                 .all()
         results = self._bonus_list(
             assignments, amount, reason, override_bonused_status)
@@ -507,7 +555,7 @@ class MTurkServicesWrapper(object):
                 result = self.bonus_local_assignment(
                     bonus_this, amount, reason, override_bonused_status)
                 results.append(result)
-            elif isinstance(bonus_this, string_types):
+            elif isinstance(bonus_this, str):
                 # assume that the str is just an assignment_id
                 assignment_id = bonus_this
                 result = self.bonus_nonlocal_assignment(
@@ -520,21 +568,23 @@ class MTurkServicesWrapper(object):
         tried_this = self._try_fetch_local_credited_assignment(
             assignment_id)
         if isinstance(tried_this, Participant):
-            result = self.bonus_local_assignment(tried_this, amount, reason, override_bonused_status=True)
+            result = self.bonus_local_assignment(tried_this, amount, reason,
+                                                 override_bonused_status=True)
             return result
         else:
             if not all_studies:
                 raise AssignmentIdNotFoundInLocalDBError(assignment_id=assignment_id)
             else:
-                response = self.bonus_nonlocal_assignment(assignment_id, amount, reason, worker_id=None)
+                response = self.bonus_nonlocal_assignment(assignment_id, amount, reason,
+                                                          worker_id=None)
                 if response.success:
-                    return True
+                    return {'success': True}
                 else:
                     raise response.exception
 
     @amt_services_wrapper_response
     def bonus_nonlocal_assignment(self, assignment_id, amount, reason, worker_id=None):
-        ''' Bonus assignment '''
+        """ Bonus assignment """
 
         '''
         If this is supposed to bonus a local_assignment, then call `bonus_local_assignment`,
@@ -547,9 +597,12 @@ class MTurkServicesWrapper(object):
         Otherwise, just give at least an assignment_id. Worker_id is nice if you have it, but if
         you don't, amt_services will look it up.
         '''
-        mode = 'sandbox' if self.sandbox else 'live'
 
         result = {}
+        try:
+            float(amount)
+        except ValueError as e:
+            raise e
         if amount <= 0:
             raise BadBonusAmountError(amount, assignment_id=assignment_id)
         if not reason:
@@ -568,7 +621,7 @@ class MTurkServicesWrapper(object):
 
     @amt_services_wrapper_response
     def tally_hits(self):
-        hits = self.get_active_hits(all_studies=False).data['active_hits']
+        hits = self.get_active_hits(all_studies=False).data
         num_hits = 0
         if hits:
             num_hits = len(hits)
@@ -584,20 +637,30 @@ class MTurkServicesWrapper(object):
     @amt_services_wrapper_response
     def get_active_hits(self, all_studies=False):
         hits = self._get_hits(all_studies)
-        active_hits = [hit for hit in hits if not hit.options['is_expired']]
-        return {'active_hits': active_hits}
+        active_hits = self._filter_active(hits)
+        return active_hits
+
+    def _filter_active(self, hits):
+        return [hit for hit in hits if not hit.options['is_expired']]
 
     @amt_services_wrapper_response
     def get_reviewable_hits(self, all_studies=False):
         hits = self._get_hits(all_studies)
-        reviewable_hits = [hit for hit in hits if hit.options['status'] == "Reviewable"
-                           or hit.options['status'] == "Reviewing"]
-        return {'reviewable_hits': reviewable_hits}
+        reviewable_hits = self._filter_reviewable(hits)
+        return reviewable_hits
+
+    def _filter_reviewable(self, hits):
+        return [hit for hit in hits if hit.options['status'] in ['Reviewable', 'Reviewing']]
 
     @amt_services_wrapper_response
     def get_all_hits(self, all_studies=False):
         hits = self._get_hits(all_studies)
-        return {'hits': hits}
+        return hits
+
+    @amt_services_wrapper_response
+    def get_hit(self, hit_id):
+        hit = self.amt_services.get_hit(hit_id).data
+        return hit
 
     def _get_hits(self, all_studies=False):
         # get all hits from amt
@@ -629,23 +692,25 @@ class MTurkServicesWrapper(object):
 
         """
 
-        response = self.amt_services.extend_hit(hit_id, assignments_increment=assignments, expiration_increment=minutes)
+        response = self.amt_services.extend_hit(hit_id, assignments_increment=assignments,
+                                                expiration_increment=minutes)
         if not response.success:
             raise response.exception
-            
-        return True
+
+        return {'success': True}
 
     @amt_services_wrapper_response
-    def delete_all_hits(self):
-        '''
+    def delete_all_hits(self, all_studies=False):
+        """
         Deletes all reviewable hits
-        '''
-        response = self.amt_services.get_all_hits()
+        """
+        response = self.get_all_hits(all_studies=all_studies)
         if not response.success:
             raise response.exception
         hits = response.data
         hit_ids = [hit.options['hitid'] for hit in hits if
-                   hit.options['status'] == "Reviewable"]
+                   hit.options['status'] == "Reviewable" and
+                   hit.options['number_submissions_needing_action'] == 0]
         results = []
         for hit_id in hit_ids:
             _result = self.delete_hit(hit_id)
@@ -654,33 +719,31 @@ class MTurkServicesWrapper(object):
 
     @amt_services_wrapper_response
     def delete_hit(self, hit_id):
-        '''
+        """
         Deletes a single hit if it is reviewable
-        '''
+        """
         # Check that the HIT is reviewable
 
         response = self.amt_services.delete_hit(hit_id)
         # self.web_services.delete_ad(hit)  # also delete the ad
         if not response.success:
-            raise response.exception
+            return {'exception': response.exception, 'hit_id': hit_id}
         else:
-            if self.sandbox:
-                success_message = "deleted sandbox HIT {}".format(hit_id)
-            else:
-                success_message = "deleted live HIT {}".format(hit_id)
-            return True
+            success_message = f"deleted {self.mode} HIT {hit_id}"
+            return {'hit_id': hit_id, 'success': True, 'message': success_message}
 
     @amt_services_wrapper_response
     def expire_hit(self, hit_id):
         response = self.amt_services.expire_hit(hit_id)
         if not response.success:
-            raise response.exception
+            raise AmtServicesWrapperError(
+                'Error expiring hit {}'.format(hit_id)) from response.exception
         else:
-            return True
+            return {'hit_id': hit_id, 'success': True}
 
     @amt_services_wrapper_response
     def expire_all_hits(self):
-        hits_data = self.get_active_hits().data['active_hits']
+        hits_data = self.get_active_hits().data
         hit_ids = [hit.options['hitid'] for hit in hits_data]
         results = []
         for hit_id in hit_ids:
@@ -688,139 +751,94 @@ class MTurkServicesWrapper(object):
         return {'results': results}
 
     @amt_services_wrapper_response
-    def create_hit(self, num_workers, reward, duration):
-        ''' Create a HIT '''
-        if not num_workers or not reward or not duration:
-            raise MissingArgumentsError()
+    def create_hit(self, num_workers, reward, duration, whitelist_qualification_ids=None,
+                   blacklist_qualification_ids=None):
+        """
+        Create a HIT
+
+        `whitelist_qualification_ids` and `blacklist_qualification_ids` get extended
+        with any values set in the config
+        """
+        if whitelist_qualification_ids is None:
+            whitelist_qualification_ids = []
+        if blacklist_qualification_ids is None:
+            blacklist_qualification_ids = []
 
-        if self.sandbox:
-            mode = 'sandbox'
-        else:
-            mode = 'live'
         server_loc = str(self.config.get('Server Parameters', 'host'))
 
-        use_psiturk_ad_server = self.config.getboolean(
-            'Shell Parameters', 'use_psiturk_ad_server')
-
-        if use_psiturk_ad_server:
-            if not self.web_services.check_credentials():
-                raise InvalidPsiturkCredentialsError()
-
         if not self.amt_services.verify_aws_login():
             raise InvalidAWSCredentialsError()
 
-        ad_id = None
-        if use_psiturk_ad_server:
-            create_ad_id_response = self.create_psiturk_ad()
-            if not create_ad_id_response.success:
-                raise create_ad_id_response.exception
-            else:
-                ad_id = create_ad_id_response.data['ad_id']
-            if ad_id:
-                ad_location = self.web_services.get_ad_url(
-                    ad_id, int(self.sandbox))
-                hit_config = self._generate_hit_config(
-                    ad_location, num_workers, reward, duration)
-                response = self.amt_services.create_hit(hit_config)
-                if not response.success:
-                    raise response.exception
-                else:
-                    hit_id = response.data['HITId']
-                    if not self.web_services.set_ad_hitid(ad_id, hit_id, int(self.sandbox)):
-                        raise AdPsiturkOrgError(message="Unable to update Ad on http://ad.psiturk.org to point at HIT.")
-            else:
-                create_failed = True
-                raise AdPsiturkOrgError(message="Unable to create Ad on http://ad.psiturk.org.")
-
-        else:  # not using psiturk ad server
-            ad_location = "{}?mode={}".format(self.config.get(
-                'Shell Parameters', 'ad_location'), mode)
-            hit_config = self._generate_hit_config(
-                ad_location, num_workers, reward, duration)
-            response = self.amt_services.create_hit(hit_config)
-            if not response.success:
-                raise response.exception
-            else:
-                hit_id = response.data['HITId']
+        ad_url = self.config.get_ad_url()
+        ad_url = f"{ad_url}?mode={self.mode}"
+        hit_config = self._generate_hit_config(
+            ad_url, num_workers, reward, duration, whitelist_qualification_ids, blacklist_qualification_ids)
+        response = self.amt_services.create_hit(hit_config)
+        if not response.success:
+            raise response.exception
+        else:
+            hit_id = response.data['HITId']
 
         # stash hit id in psiturk database
         hit = Hit(hitid=hit_id)
         db_session.add(hit)
         db_session.commit()
 
-        return {'hit_id': hit_id, 'ad_id': ad_id}
+        return {'hit_id': hit_id}
 
     @amt_services_wrapper_response
-    def create_psiturk_ad(self):
-        # register with the ad server (psiturk.org/ad/register) using POST
-        if os.path.exists('templates/ad.html'):
-            ad_html = open('templates/ad.html').read()
+    def list_qualification_types(self, *args, **kwargs):
+        """
+        client.list_qualification_types(
+            Query='string',
+            MustBeRequestable=True|False,
+            MustBeOwnedByCaller=True|False,
+            NextToken='string',
+            MaxResults=123
+        )
+
+        To just get the ones created by le user, call:
+
+            list_qualification_types(MustBeOwnedByCaller=True)
+        """
+        response = self.amt_services.list_qualification_types(*args, **kwargs)
+        if not response.success:
+            raise response.exception
         else:
-            raise AdHtmlNotFoundError()
+            qualification_types = response.data
+            return {'qualification_types': qualification_types}
 
-        size_of_ad = sys.getsizeof(ad_html)
-        if size_of_ad >= 1048576:
-            raise AdHtmlTooLarge(size_of_ad)
+    def _generate_hit_config(self, ad_url, num_workers, reward, duration, whitelist_qualification_ids=None, blacklist_qualification_ids=None):
+        if whitelist_qualification_ids is None:
+            whitelist_qualification_ids = []
 
-        # what all do we need to send to server?
-        # 1. server
-        # 2. port
-        # 3. support_ie?
-        # 4. ad.html template
-        # 5. contact_email in case an error happens
+        if blacklist_qualification_ids is None:
+            blacklist_qualification_ids = []
 
-        if self.config.has_option('Server Parameters', 'adserver_revproxy_host'):
-            # misnomer, could actually be a fqdn sans protocol
-            ip_address = self.config.get(
-                'Server Parameters', 'adserver_revproxy_host')
-            if self.config.has_option('Server Parameters', 'adserver_revproxy_port'):
-                port = self.config.getint(
-                    'Server Parameters', 'adserver_revproxy_port')
-            else:
-                port = 80
-        else:
-            ip_address = str(get_my_ip())
-            port = str(self.config.get('Server Parameters', 'port'))
+        require_quals = self.config.get('HIT Configuration', 'require_quals', fallback=None)
+        if require_quals:
+            whitelist_qualification_ids.extend(require_quals.split(','))
 
-        if self.config.has_option('HIT Configuration', 'allow_repeats'):
-            allow_repeats = self.config.getboolean(
-                'HIT Configuration', 'allow_repeats')
-        else:
-            allow_repeats = False
+        block_quals = self.config.get('HIT Configuration', 'block_quals', fallback=None)
+        if block_quals:
+            blacklist_qualification_ids.extend(block_quals.split(','))
 
-        ad_content = {
-            'psiturk_external': True,
-            'server': ip_address,
-            'port': port,
-            'browser_exclude_rule': str(self.config.get('HIT Configuration', 'browser_exclude_rule')),
-            'is_sandbox': int(self.sandbox),
-            'ad_html': ad_html,
-            # 'amt_hit_id': hitid, Don't know this yet
-            'organization_name': str(self.config.get('HIT Configuration', 'organization_name')),
-            'experiment_name': str(self.config.get('HIT Configuration', 'title', raw=True)),
-            'contact_email_on_error': str(self.config.get('HIT Configuration', 'contact_email_on_error')),
-            'ad_group': str(self.config.get('HIT Configuration', 'ad_group')),
-            'keywords': str(self.config.get('HIT Configuration', 'psiturk_keywords')),
-            'allow_repeats': int(allow_repeats)
-        }
-        ad_id = self.web_services.create_ad(ad_content)
-        if not ad_id:
-            raise AdPsiturkOrgError(message='Error creating the ad.')
-        return {'ad_id': ad_id}
-
-    def _generate_hit_config(self, ad_location, num_workers, reward, duration):
         hit_config = {
-            "ad_location": ad_location,
+            "ad_location": ad_url,
             "approve_requirement": self.config.getint('HIT Configuration', 'Approve_Requirement'),
             "us_only": self.config.getboolean('HIT Configuration', 'US_only'),
-            "lifetime": datetime.timedelta(hours=self.config.getfloat('HIT Configuration', 'lifetime')),
+            "lifetime": datetime.timedelta(
+                hours=self.config.getfloat('HIT Configuration', 'lifetime')),
             "max_assignments": num_workers,
             "title": self.config.get('HIT Configuration', 'title', raw=True),
             "description": self.config.get('HIT Configuration', 'description', raw=True),
-            "keywords": self.config.get('HIT Configuration', 'amt_keywords'),
+            "keywords": self.config.get('HIT Configuration', 'keywords'),
             "reward": reward,
             "duration": datetime.timedelta(hours=duration),
             "number_hits_approved": self.config.getint('HIT Configuration', 'number_hits_approved'),
-            "require_master_workers": self.config.getboolean('HIT Configuration', 'require_master_workers')
+            "require_master_workers": self.config.getboolean('HIT Configuration',
+                                                             'require_master_workers'),
+            "whitelist_qualification_ids": whitelist_qualification_ids,
+            "blacklist_qualification_ids": blacklist_qualification_ids
         }
         return hit_config
